#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Script to test Quran.com API translation IDs to find the correct one for Dutch.
It fetches a specific verse (e.g., 4:34) using translation IDs from 1 to 100
and prints the result for each ID.
"""

import requests
import time
import sys
import html
import textwrap

# --- Configuration ---
API_BASE_URL = "https://api.quran.com/api/v4"
VERSE_KEY = "4:34"  # A verse likely to have distinct translations
START_ID = 100
END_ID = 200
REQUEST_DELAY_SECONDS = 0.5 # Delay between API requests to avoid rate limiting
TERMINAL_WIDTH = 80

# --- Functions ---

def get_verse_by_translation_id(verse_key: str, translation_id: int) -> str | None:
    """
    Fetches a specific verse translation using a given translation ID.

    Args:
        verse_key: The verse key (e.g., "4:34").
        translation_id: The translation ID to test.

    Returns:
        The translated text as a string, or None if an error occurred or no text found.
    """
    verse_url = f"{API_BASE_URL}/verses/by_key/{verse_key}"
    params = {
        'translations': str(translation_id),
        'fields': 'text_imlaei' # Requesting text field for translations
    }
    headers = {'Accept': 'application/json'}

    try:
        response = requests.get(verse_url, params=params, headers=headers, timeout=10)
        response.raise_for_status() # Check for HTTP errors
        data = response.json()

        verse_data = data.get('verse', {})
        api_translations = verse_data.get('translations', [])

        if api_translations:
            # Find the translation matching the requested ID
            for trans in api_translations:
                if trans.get('resource_id') == translation_id:
                    raw_text = html.unescape(trans.get('text', ''))
                    # Basic cleaning: remove common HTML tags, keep newlines
                    clean_text = ''.join(c for c in raw_text.replace('<br>', '\n').replace('</br>', '').replace('<sup>', '[').replace('</sup>', ']') if c.isprintable() or c.isspace())
                    return clean_text.strip()
        return "[No text found for this ID]" # Return specific message if translation exists but text is empty

    except requests.exceptions.Timeout:
        print(f"  [ID {translation_id}] Error: Request timed out.", file=sys.stderr)
        return None
    except requests.exceptions.HTTPError as e:
        # Handle 404 Not Found specifically - likely means the ID is invalid
        if e.response.status_code == 404:
             # print(f"  [ID {translation_id}] Info: Translation ID not found (404).", file=sys.stderr)
             return "[ID not found (404)]"
        else:
            print(f"  [ID {translation_id}] Error: HTTP Error: {e.response.status_code} {e.response.reason}", file=sys.stderr)
        return None
    except requests.exceptions.RequestException as e:
        print(f"  [ID {translation_id}] Error: Request failed: {e}", file=sys.stderr)
        return None
    except requests.exceptions.JSONDecodeError:
        print(f"  [ID {translation_id}] Error: Could not decode JSON response.", file=sys.stderr)
        return None

def main():
    """
    Main function to loop through IDs and print results.
    """
    print(f"Attempting to find Dutch translation for verse {VERSE_KEY} by testing IDs {START_ID}-{END_ID}...")
    print("-" * TERMINAL_WIDTH)

    found_potential = False
    text_wrapper = textwrap.TextWrapper(width=TERMINAL_WIDTH - 4, initial_indent="    ", subsequent_indent="    ")

    for i in range(START_ID, END_ID + 1):
        print(f"Testing ID: {i}")
        translation_text = get_verse_by_translation_id(VERSE_KEY, i)

        if translation_text is not None:
            # Don't print the full text if it's just an error message from the function
            if translation_text.startswith("[") and translation_text.endswith("]"):
                 print(f"  Result: {translation_text}")
            else:
                print("  Text:")
                wrapped_text = text_wrapper.fill(translation_text)
                print(wrapped_text)
                found_potential = True # Mark that we found some actual text
        else:
            # Error messages are printed within the get_verse function
            pass

        print("-" * (TERMINAL_WIDTH // 2))

        # Pause between requests
        time.sleep(REQUEST_DELAY_SECONDS)

    print("-" * TERMINAL_WIDTH)
    if found_potential:
        print(f"Finished testing IDs {START_ID}-{END_ID}. Please review the output above to identify the Dutch translation.")
    else:
        print(f"Finished testing IDs {START_ID}-{END_ID}. No translation text was successfully retrieved. Check API or network.")

if __name__ == "__main__":
    main()
